<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\FpaTabelafolha;
use Folha\Entities\FpaFaixatabela;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\FPA\Repository\TabelaFolhaRepository;
use HCM\FPA\Repository\TabelaFaixaRepository;

class TabelaFolhaService {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $tabelaFolhaRepository;
    protected $tabelaFaixaRepository;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, TabelaFolhaRepository $tabelaFolhaRepository, TabelaFaixaRepository $tabelaFaixaRepository) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->tabelaFolhaRepository = $tabelaFolhaRepository;
        $this->tabelaFaixaRepository = $tabelaFaixaRepository; 
    }
    
    public function deleteTabelaFolha($dataset) {
        
        $nrorg = $dataset["NRORG"];
        $nrtabelafolha = $dataset["NRTABELAFOLHA"];
        $faixatabela = FpaFaixatabela::findByNrtabelafolha($nrtabelafolha);
        if (isset($faixatabela[0])) {
            throw new \Exception(Messages::HCM05000_FPA_TABELAFOLHA_EXISTS_FAIXATABELA_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $tabelafolha = FpaTabelafolha::findOneByNrtabelafolhaAndNrorg($nrtabelafolha, $nrorg);
            $this->entityManager->remove($tabelafolha);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    /**
     * Essa função faz um pré-preparo dos argumentos a serem utilizadas na query, situada
     * no Repository
     */
    public function reajustaValores($dataSet) {
        $DTINIVIGENCIA = $dataSet['DTINIVIGENCIA'];
        $NRTABELAFOLHA = $dataSet['NRTABELAFOLHA'];
        $vlrReajuste = $dataSet['Reajuste'];
        $porcentagemReajuste = $vlrReajuste/100;
        $this->tabelaFaixaRepository->reajustaValores($NRTABELAFOLHA, $DTINIVIGENCIA, $porcentagemReajuste);
    }
    
    /**
     * Essa função é responsável por chamar outras duas: uma para a duplicação do registro
     * folha; e outra para a inserção dos registros faixas no registro folha duplicado
     */
    public function replicarFolhaFaixas($row){
        $auditoria = $this->setaAuditoria();
        $this->tabelaFolhaRepository->replicarFolha($auditoria, $row);
        $faixas = $this->tabelaFaixaRepository->obterFaixas($row);
        $this->tabelaFaixaRepository->replicarFaixas($auditoria, $row, $faixas);
    }
    
    public function setaAuditoria(){
        $DTINCLUSAO = $this->hcmParameters->getDataAtual();
        $CDOPERINCLUSAO = $this->hcmParameters->getCdoperador();
        $NRORGINCLUSAO = $this->hcmParameters->getNrorgTrabalho();
        $NRORG = $this->hcmParameters->getNrorgTrabalho();
        $auditoria = [
            "DTINCLUSAO" => $DTINCLUSAO,
            "CDOPERINCLUSAO" => $CDOPERINCLUSAO,
            "NRORGINCLUSAO" => $NRORGINCLUSAO,
            "NRORG" => $NRORG,
        ];
        return $auditoria;
    }
}