<?php

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\AlteOcupacaoService;
use HCM\Geral\Services\ESocialService;
use HCM\Util\DateUtil;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class AlteOcupacaoController extends Crud {

    protected $dataSourceName = 'gpe_alteocupacao';
    protected $dataSourceNameGet = 'getAlteocupacao';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_RESCISAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AlteOcupacaoService $alteOcupacaoService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->alteOcupacaoService = $alteOcupacaoService;
        $this->eSocialService = $eSocialService;
        parent::__construct($dataSourceManager);
    }

    public function getAlteocupacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setAlteocupacao (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrorg = $row['NRORG'];
                $nrvinculom = $row['NRVINCULOM'];
                $nrocupacaom = $row['NROCUPACAOM'];
                $dtiniocupacao = $row['DTINIOCUPACAO'];
                $dtfimocupacao = $row['DTFIMOCUPACAO'];

                if ($row['__is_new']) {
                    // Valida se já existe ocupacao para a mesma data
                    $this->alteOcupacaoService->beforeSave($nrorg, $nrvinculom, $dtiniocupacao);
                }

                if (array_key_exists('NRALTEOCUPACAO', $row)) {
                    if ($row['NRALTEOCUPACAO'] === '') {
                        $row['NRALTEOCUPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true);
                    }
                } else {
                    $row['NRALTEOCUPACAO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTEOCUPACAO', 1, 12, true);
                }
                if(!array_key_exists('NRMOTIVOOCUPA', $row)){
                  $row['NRMOTIVOOCUPA'] = null;
                }
                if(!array_key_exists('NRTPMODALIDSAL', $row)){
                  $row['NRTPMODALIDSAL'] = null;
                }
                if(!array_key_exists('NRFAIXA', $row)){
                  $row['NRFAIXA'] = null;
                }
                
                parent::save($request, $response);

                if ($row['__is_new']) {
                    // Verifica se o vínculo possui histórico para a competência
                    $competencCadastro = DateUtil::setDay($row['DTINIOCUPACAO'], 1);
                    $this->alteOcupacaoService->afterInsert($nrorg, $nrvinculom, $competencCadastro);
                }
                // Atualiza histórico do vínculo
                $this->alteOcupacaoService->afterSave($nrorg, $nrvinculom, $nrocupacaom, $dtiniocupacao, $dtfimocupacao, true);
                
                if(array_key_exists('IDENVIAESOCIAL', $row) && $row['IDENVIAESOCIAL'] == 'S'){
                    $this->eSocialService->envioEsocialDescentralizado($row, $response);
                }
            }
        } catch (\Exception $e) {
            if ($e->getCode() != 0) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function deleteAlteocupacao(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
            $row = $request->getDataSet()->getRows()[0];
            // Atualiza histórico do vínculo de acordo com a ocupacao anterior
            $this->alteOcupacaoService->afterDelete($row['NRORG'], $row['NRVINCULOM'], $row['DTINIOCUPACAO']);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}