<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\VinculoRepository;
use HCM\Geral\Repository\ESocialRepository;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Folha\Entities\VabLinhavinc;
use HCM\Util\Messages;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use Teknisa\Model\Gestaopessoas\Entities\GpeCondfispes;
use Teknisa\Model\Gestaopessoas\Entities\FpaTpdemissao;
use HCM\Util\Util;

class VinculoServices {

    protected $gpe_Vinculoh;
    /**
     * @var vinculoRepository
     */
    protected $vinculoRepository;
    protected $eSocialRepository;

    public function __construct(EntityManager $entityManager, VinculoRepository $vinculoRepository, ESocialRepository $eSocialRepository, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->vinculoRepository = $vinculoRepository;
        $this->eSocialRepository = $eSocialRepository;
        $this->hcmParameters = $hcmParameters;
        
        $this->gpe_Vinculoh = Repositories::GPE_VINCULOH;
        $this->gpe_Vinculom = Repositories::GPE_VINCULOM;
    }

    /**
     * @return int
     */
     
    const QRYcancelaAdmissao = "SELECT(SELECT NMTIPOMOVIMENT FROM FPA_TIPOMOVIMENT WHERE NRTIPOMOVIMENT = CF.NRTIPOMOVIMENT) || ' - ' || TO_CHAR(CF.DTOCORRENCIA, 'MM/YYYY') || ' - ' || (SELECT NMTPMODALIDCAL FROM FPA_TPMODALIDCAL WHERE NRTPMODALIDCAL = CF.NRTPMODALIDCAL ) || ' - ' || CF.NROCORRECAL SAIDA FROM FPA_CALCULOFOLHA CF JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = CF.NRORG AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA AND ICF.NRVINCULOM = :P_NRVINCULOM WHERE CF.NRORG = :P_NRORG AND CF.NRTIPOMOVIMENT IN ( 1, 2, 4, 10, 17, 19, 21) GROUP BY CF.DTOCORRENCIA, CF.NRTIPOMOVIMENT, CF.NRTPMODALIDCAL, CF.NROCORRECAL ORDER BY CF.DTOCORRENCIA, CF.NRTIPOMOVIMENT, CF.NRTPMODALIDCAL, CF.NROCORRECAL";
    
    public function getNrVinculoM($nrorg) {
        try{
            $result = $this->vinculoRepository->getNrVinculoM($nrorg);
            $id = $result[0]['MAX_ID'];
            return !is_null($id) ? ++$id : 1;
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }
    }

     public function getNrVinculoH($nrorg) {
        try{
            $result = $this->vinculoRepository->getNrVinculoH($nrorg);
            $id = $result[0]['MAX_ID'];
            return !is_null($id) ? ++$id : 1;
        }
        catch (Exception $e) {
            throw new exception ($e->getMessage());
        }
    }
    
    public function novoHistorico($nrorg, $nrvinculom, $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
                                  $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao,
                                  $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                                  $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin,
                                  $nrestrutsind, $confidencial, $cdmatriculaEsocial = null, $idpagpropfer13 = null, $idAssocSindicato = 'N')
    {
        /** @var \Teknisa\Model\Gestaopessoas\Entities\GpeVinculoh $vinculoh */
        $vinculoh = new $this->gpe_Vinculoh;

        $vinculoh->setNrorg($nrorg);
        $vinculoh->setNrvinculom($nrvinculom);
        $vinculoh->setDtmescompetenc($dtmescompetenc);

        $vinculoh->setDtbaseferias($dtbaseferias);
        $vinculoh->setIdmultvinc($idmultvinc);
        $vinculoh->setNrsitufuncm($nrsitufuncm);
        $vinculoh->setNrtpmovtransfm($nrtpmovtransfm);
        $vinculoh->setCdcontribindividual($cdcontribindividual);
        $vinculoh->setDtfimcontrdetermin($dtfimcontrdetermin);

        $vinculoh->setNrcargo($nrcargo);
        $vinculoh->setNrfuncao($nrfuncao);
        $vinculoh->setNrescalatrabm($nrescalatrabm);
        $vinculoh->setIdremuneracao($idremuneracao);
        $vinculoh->setIdtppagamento($idtppagamento);
        $vinculoh->setNrtpmodalidsal($nrtpmodalidsal);

        $vinculoh->setNrdependir($nrdependir);
        $vinculoh->setNrdependsfam($nrdependsfam);
        $vinculoh->setConfidencial($confidencial);

        $vinculoh->setNrestrutlegal($nrestrutlegal);
        $vinculoh->setNrestrutgeren($nrestrutgeren);
        $vinculoh->setNrestrutsind($nrestrutsind);
        $vinculoh->setIdcontribuisind(($idcontribuisind) ? : 'NUNCA');
        $vinculoh->setCdmatriculaesocial($cdmatriculaEsocial);
        $vinculoh->setIdpagpropfer13($idpagpropfer13);
        $vinculoh->setIdassocsindicato($idAssocSindicato);

        $vinculoh->setDtinclusao(DateUtil::getDataAtual());
        $vinculoh->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $vinculoh->setDtultatu(DateUtil::getDataAtual());
        $vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        $vinculoh->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());

        $vinculoh->setNrvinculoh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOH', 1, 12, true));
        
        $this->entityManager->persist($vinculoh);
        $this->entityManager->flush($vinculoh);
        
        return $vinculoh;
    }

    /**
     * Verifica se o vínculo possui histórico mais novo que a competencia atual.
     * Caso não tenha gera novo histórico na competência atual.
     *
     * @param $nrorg
     * @param $nrvinculom
     * @throws \Exception
     */
    public function verificaHistoricoCompetencia($nrorg, $nrvinculom, $competencia = null) {
        if ($competencia == null){
            $competencia = $this->hcmParameters->getDtCompetencia();
    
            $gpevinculoh = $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia);
            $dthistorico = DateUtil::getDataDeString($gpevinculoh->getDtmescompetenc(), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtcompetenc = DateUtil::getDataDeString($competencia);
            $dtcompetenc->setTime(0,0,0);
            if (DateUtil::comparaDatas($dthistorico, $dtcompetenc)) {
                $this->novoHistorico($nrorg, $nrvinculom, $dtcompetenc, $gpevinculoh->getNrsitufuncm(), $gpevinculoh->getNrcargo(), $gpevinculoh->getNrescalatrabm(),
                    $gpevinculoh->getIdcontribuisind(), $gpevinculoh->getNrdependir(), $gpevinculoh->getNrdependsfam(), $gpevinculoh->getDtbaseferias(), $gpevinculoh->getIdremuneracao(),
                    $gpevinculoh->getIdtppagamento(), $gpevinculoh->getNrestrutlegal(), $gpevinculoh->getNrestrutgeren(), $gpevinculoh->getNrtpmodalidsal(), $gpevinculoh->getIdmultvinc(),
                    $gpevinculoh->getNrfuncao(), $gpevinculoh->getCdcontribindividual(), $gpevinculoh->getNrtpmovtransfm(), $gpevinculoh->getDtfimcontrdetermin(),
                    $gpevinculoh->getNrestrutsind(), $gpevinculoh->getConfidencial(), $gpevinculoh->getCdmatriculaesocial(), $gpevinculoh->getIdpagpropfer13(), $gpevinculoh->getIdassocsindicato());
            }
        } else {
            $vinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom, "nrorg" => $nrorg));
            //$gpevinculoh = $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia);
            $gpevinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $competencia->format('d/m/Y'));
            $dthistorico = DateUtil::getDataDeString($gpevinculoh->getDtmescompetenc(), DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtcompetenc = DateUtil::getDataDeString($competencia);
            $dtcompetenc->setTime(0,0,0);
            if (DateUtil::comparaDatas($dthistorico, $dtcompetenc)) {
                $this->novoHistorico($nrorg, $nrvinculom, $dtcompetenc, $gpevinculoh->getNrsitufuncm(), $gpevinculoh->getNrcargo(), $gpevinculoh->getNrescalatrabm(),
                    $gpevinculoh->getIdcontribuisind(), $gpevinculoh->getNrdependir(), $gpevinculoh->getNrdependsfam(), $gpevinculoh->getDtbaseferias(), $gpevinculoh->getIdremuneracao(),
                    $gpevinculoh->getIdtppagamento(), $gpevinculoh->getNrestrutlegal(), $gpevinculoh->getNrestrutgeren(), $gpevinculoh->getNrtpmodalidsal(), $gpevinculoh->getIdmultvinc(),
                    $gpevinculoh->getNrfuncao(), $gpevinculoh->getCdcontribindividual(), $gpevinculoh->getNrtpmovtransfm(), $gpevinculoh->getDtfimcontrdetermin(),
                    $gpevinculoh->getNrestrutsind(), $gpevinculoh->getConfidencial(), $gpevinculoh->getCdmatriculaesocial(), $gpevinculoh->getIdpagpropfer13(), $gpevinculoh->getIdassocsindicato());
            }
            
        }
    }

    /**
     * Retorna histórico do vínculo na competência informada ou na competência atual
     *
     * @param $nrorg
     * @param $nrvinculom
     * @param null $dtmescompetenc
     * @return mixed
     */
    public function retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc = null) {
        if (!$dtmescompetenc) {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        }
        return $this->vinculoRepository->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtmescompetenc);
    }
    
    public function retornaHistoricoAnteriorUltimoHistorico($nrorg, $nrvinculom, $nrvinculoh, $dtmescompetenc = null) {
        if (!$dtmescompetenc) {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        }
        return $this->vinculoRepository->retornaHistoricoAnteriorUltimoHistorico($nrorg, $nrvinculom, $nrvinculoh, $dtmescompetenc);
    }
    
    /**
     * Retorna todos os históricos a partir da comptência informada
     */
    public function retornaHistoricosPosteriores($nrorg, $nrvinculom, $dtmescompetenc) {
        $where   = "NRORG = :NRORG AND NRVINCULOM = :NRVINCULOM AND DTMESCOMPETENC > :DTMESCOMPETENC";
    	$params  = array(":NRORG" => $nrorg, ":NRVINCULOM" => $nrvinculom, ":DTMESCOMPETENC" => $dtmescompetenc);
    	$orderby = "DTMESCOMPETENC ASC";
    	return \Folha\Entities\GpeVinculoh::getObjectsByWhere($where, $params, $orderby);
    }
    
    /**
     * Atualiza o histórico de vinculo via lista (array) de nrvinculoh.
     */
    public function atualizaHistoricosVinculo($nrorg, $listaNrvinculoh,
        $nrsitufuncm, $nrcargo, $nrfuncao, $nrescalatrabm, $nrestrutlegal, $nrestrutgeren, $nrestrutsind, $cdmatriculaesocial)
    {
    	try {
    	    $this->entityManager->beginTransaction();
    	    
        	$where      = array("nrorg" => $nrorg, "nrvinculoh" => $listaNrvinculoh);
        	$historicos = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findBy($where);
    	    $msg        = array(); //Mensagens das competências alteradas
            
            foreach($historicos as $historico) {
                if($nrsitufuncm  ) $historico->setNrsitufuncm(  $nrsitufuncm  );
        		if($nrcargo      ) $historico->setNrcargo(      $nrcargo      );
        		if($nrfuncao     ) $historico->setNrfuncao(     $nrfuncao     );
        		if($nrescalatrabm) $historico->setNrescalatrabm($nrescalatrabm);
        		if($nrestrutlegal) $historico->setNrestrutlegal($nrestrutlegal);
        		if($nrestrutgeren) $historico->setNrestrutgeren($nrestrutgeren);
        		if($nrestrutsind ) $historico->setNrestrutsind( $nrestrutsind );
        		if($cdmatriculaesocial) $historico->setCdmatriculaesocial($cdmatriculaesocial);
        		
        		$historico->setNrorgultatu( $nrorg                               );
        		$historico->setCdoperultatu($this->hcmParameters->getCdoperador());
        		$historico->setDtultatu(    DateUtil::getDataAtual()             );
                
        		$msg[] = $historico->getDtmescompetenc()->format('d/m/Y');
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return $msg;
    	} catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }

    // retorna somente vínculo ativos na competência
    public function retornaVinculosAtivosCompetencia($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrtipovinculom) {
        return $this->vinculoRepository->retornaVinculosAtivosCompetencia($nrorg, $dtmescompetenc, $nrestrutlegal, $nrestrutgeren, $nrtipovinculom);
    }

    // retorna vinculos ativos na competência parametrizado
    public function retornaVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos = false, $ds_inicio = null, $ds_fim = null, $dsOrdem = null, $carregaHistorico = false, $nrEstrutura = null, $verificaConfidencial = true, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $nrEstrutSind = null, $paramCaged = false, $idapenasdesemp = null) {
        $ds_inicio = !is_array($ds_inicio) ? trim(strtoupper($ds_inicio)) : $ds_inicio;
        $ds_fim    = !is_array($ds_fim) ? trim(strtoupper($ds_fim)) : $ds_fim;
        
        // define primeiro dia do mês competência
        $primeiroDiaAno = DateUtil::subtraiIntervalo(substr($this->hcmParameters->getPrimeiroDiaDoAno(), 0, 10), 1, DateUtil::ANO)->format('d/m/Y');
        $mesCompetencia = substr($mesCompetencia, 0, 10);
        $dtUltDiaMes = substr(DateUtil::getUltimoDiaDoMes($mesCompetencia), 0, 10);
        return $this->vinculoRepository->retornaVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos, $primeiroDiaAno, $dtUltDiaMes, $ds_inicio, $ds_fim, $dsOrdem, $carregaHistorico, $nrEstrutura, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind, $paramCaged, $idapenasdesemp);   

    }

    public function retornaRescinditosCompetencia($nrOrg, $nrOrgPadrao, $mesCompetencia, $nrEstrutura = null, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $carregaHistorico = false, $filters,$todosRescindidos,$ds_inicio = null, $ds_fim = null, $dsOrdem = null, $nrEstrutSind = null, $conditions = null, $movTransfSaida = null) {        
        $primeiroDiaAno = $this->hcmParameters->getPrimeiroDiaDoAno();
        $dtUltDiaMes = $this->hcmParameters->getUltimoDiaMes();
        
        return $this->vinculoRepository->retornaRescinditosCompetencia($nrOrg, $nrOrgPadrao, $mesCompetencia, $dtUltDiaMes, $primeiroDiaAno, $carregaHistorico, $nrEstrutura, $nrTipoVinculom, $nrVinculoEmpreg, $filters, $ds_inicio, $ds_fim, $dsOrdem, $nrEstrutSind, $todosRescindidos, $conditions, $movTransfSaida);
    }
    
    public function retornaRescindidosGeral($mesCompetencia, $ds_inicio = null, $ds_fim = null, $dtini = null, $dtfim = null, $dsOrdem = null, $nrEstrutura = null, $carregaHistorico = null, $nrTipoVinculom = null,$nrVinculoEmpreg = null, $nrEstrutSind = null, $conditions) {        
       
        return $this->vinculoRepository->retornaRescindidosGeral($mesCompetencia, $ds_inicio, $ds_fim, $dtini, $dtfim, $dsOrdem, $nrEstrutura, $carregaHistorico, $nrTipoVinculom,$nrVinculoEmpreg, $nrEstrutSind, $conditions);
    }

    public function verificaCalculoMovimentoVinculo($nrvinculom, $nrOrg, $mesCompetencia, $nrtpmodalidcal, $ocorrecal, $nrtipomoviment) {
        return $this->vinculoRepository->verificaCalculoMovimentoVinculo($nrvinculom, $nrOrg, $mesCompetencia, $nrtpmodalidcal, $ocorrecal, $nrtipomoviment);
    }
    
    public function cancelaAdmissao($row) {
        try {
            $this->entityManager->beginTransaction();
            
            //variavel que armazena as mensagens de erro das validações de item de cálculo e de envio dos eventos para o eSocial
            $mensagem = "";



            $calculos = $this->entityManager->getConnection()->prepare(self::QRYcancelaAdmissao);
            $calculos->execute(
                 array (
                        "P_NRORG"                 => $row['NRORG'],
                        "P_NRVINCULOM"            => $row['NRVINCULOM'] 
                )
            );
            $calculos = $calculos->fetchAll();
            $aux = array();
            foreach($calculos as $val) {
                $aux[] = $val['SAIDA'];
            }
            if (count($calculos) > 0) {
                $calculos = implode($aux, "<br>");
                $mensagem .= Messages::evaluateMessage(Messages::HCM00000_GER_VINCULO_POSSUI_ITENSDECALC_ERROR, $calculos);
            }
        
            //Realiza a verificação do envio da admissão para o eSocial
            if(!$this->eSocialRepository->verififcaSeAdmissaoFoiEnviadaParaOESocial($row['NRORG'], $row['NRVINCULOM'])){
                if(!empty($mensagem)){
                    $mensagem .= "<br /><br />" . Messages::HCM00048_GER_EVENTO_ADMISSAO_ENVIADO_TEXTO_COMPLEMENTAR_AVISO;
                } else {
                    $mensagem .= Messages::HCM00048_GER_EVENTO_ADMISSAO_ENVIADO;
                }
            }
            
            if(!empty($mensagem)){
                throw new \Exception($mensagem, 10);
            }
            
            $historicos = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->findBy(array("nrvinculom" => $row['NRVINCULOM'], "nrorg" => $row['NRORG']));
            foreach ($historicos as $gpevinculoh) {
                $this->entityManager->remove($gpevinculoh);
            }
            $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $row['NRVINCULOM'],"nrorg" => $row['NRORG']));
            $gpevinculom->setNrorgultatu($row['NRORG']);
            $gpevinculom->setDtrescisaovinc($gpevinculom->getDtadmissaovinc());
            $gpevinculom->setCdoperultatu($this->hcmParameters->getCdoperador());
            $gpevinculom->setDtultatu(DateUtil::getDataAtual());
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch (\Exception $e) {            
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function novoMestre(
        $nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao = null, $matricula = null, $tipoVinculo = null,
        $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null, $dtPrimeiraAdmissao = null, $dtOpcaoFGTS = null,
        $dtExameMedico = null, $dtAposentadoriaFGTS = null, $nrDiasContratoExp = null, $nrDiasContratoExpPro = null, 
        $dslivroadmissao = null, $dtentregaepi = null, $dtvenccontrexper = null, $nrclausassecurat = null,
        $nrmotivoadmissao = null, $nrvinculomsubst = null
    ){
        try{
            $vinculo = new $this->gpe_Vinculom;
            $vinculo->setNrvinculom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_VINCULOM', 1, 12, true));
            
            if(empty($matricula)){
                $matricula = $vinculo->getNrvinculom();
            }
        
            $vinculo->setNrpessoa($nrpessoa);
            $vinculo->setNrorg($nrorgTrab);
            $vinculo->setDtadmissaovinc($dtAdmissao);
            $vinculo->setDtrescisaovinc($dtRescisao);
            $vinculo->setCdmatricula($matricula);
            $vinculo->setNrtipovinculom($tipoVinculo);
            $vinculo->setNrvinculoempreg($nrvinculoempreg);
            $vinculo->setNrtpadmissao($nrtpadmissao);
            $vinculo->setNmvinculom($nmvinculom);
            $vinculo->setDtinclusao(DateUtil::getDataAtual());
            $vinculo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $vinculo->setDtultatu(DateUtil::getDataAtual());
            $vinculo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $vinculo->setCdoperultatu($this->hcmParameters->getCdoperador());
            $vinculo->setDtprimadmissao($dtPrimeiraAdmissao);
            $vinculo->setDtopcaofgts($dtOpcaoFGTS);
            $vinculo->setDtaposentafgts($dtAposentadoriaFGTS);
            $vinculo->setNrdiascontratoexp($nrDiasContratoExp);
            $vinculo->setNrdiascontratoexppro($nrDiasContratoExpPro);
            $vinculo->setDtexamemedico($dtExameMedico);
            $vinculo->setDslivroadmissao($dslivroadmissao);
            $vinculo->setDtentregaepi($dtentregaepi);
            $vinculo->setDtvenccontrexper(DateUtil::getDataDeString($dtvenccontrexper));
            $vinculo->setNrclausassecurat($nrclausassecurat);
            $vinculo->setNrmotivoadmissao($nrmotivoadmissao);
            $vinculo->setNrvinculomsubst($nrvinculomsubst);
            
        
            $this->entityManager->persist($vinculo);
            $this->entityManager->flush($vinculo);
            
            return $vinculo;
        }
        catch (Exception $e){
            $e = new Exception('Erro ao criar um novo vinculo. Nao pode inserir um novo mestre.');
            throw $e;
        }
    }
    
    public function novo($nrPessoa, $dtAdmissaoVinculo, $cdMatricula, $nrTipoVinculom, $nrTipoAdmissao,
                         $nrVinculoEmpregaticio, $dtOpcaoFGTS, $dtAposentadoriaFGTS, $dtPrimeiraAdmissao,$nmVinculom,
                         $nrDiasContratoExp, $nrDiasContratoExpPro, $dtExameMedico, $dtmescompetenc, $nrsitufuncm,
                         $nrcargo, $nrescalatrabm, $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias,
                         $idremuneracao, $idtppagamento, $nrestrutlegal, $nrestrutgeren, $nrtpmodalidsal, $idmultvinc,
                         $nrfuncao, $cdcontribindividual, $nrtpmovtransfm, $dtfimcontrdetermin, $nrestrutsind,
                         $confidencial = null,$nrorg = null,$dtRescisao = null, $dslivroadmissao = null, $dtentregaepi = null, 
                         $cdmatriculaEsocial = null, $dtvenccontrexper = null, $nrclausassecurat = null, $idpagpropfer13 = null,
                         $nrmotivoadmissao = null, $nrvinculomsubst = null){
        
                             
        $vinculom = $this->novoMestre($nrPessoa, $nrorg, $dtAdmissaoVinculo, $dtRescisao, $cdMatricula, $nrTipoVinculom,
        $nrVinculoEmpregaticio, $nrTipoAdmissao, $nmVinculom, $dtPrimeiraAdmissao, $dtOpcaoFGTS,
        $dtExameMedico, $dtAposentadoriaFGTS, $nrDiasContratoExp, $nrDiasContratoExpPro, $dslivroadmissao, $dtentregaepi,
        $dtvenccontrexper, $nrclausassecurat, $nrmotivoadmissao, $nrvinculomsubst);
        
        $vinculoh = $this->novoHistorico(
            $nrorg,$vinculom->getNrvinculom(), $dtmescompetenc, $nrsitufuncm, $nrcargo, $nrescalatrabm,
            $idcontribuisind, $nrdependir, $nrdependsfam, $dtbaseferias, $idremuneracao, $idtppagamento, $nrestrutlegal,
            $nrestrutgeren, $nrtpmodalidsal, $idmultvinc, $nrfuncao, $cdcontribindividual, $nrtpmovtransfm,
            $dtfimcontrdetermin, $nrestrutsind, $confidencial, $cdmatriculaEsocial, $idpagpropfer13
        );
        return array($vinculom, $vinculoh);
        
    }
    
    public function verificaSituaçãoVinculoRescisao($nrorg,$nrpessoa,$dtrescisao){
        
        
        $pessoah = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->findOneBy(array('nrorg' => $nrorg,
                    'nrpessoa' => $nrpessoa),(array('dtmescompetenc' => 'DESC')));
    
        if ($pessoah->getNrcondfispes() && ($pessoah->getNrcondfispes() != GpeCondfispes::CONDFISPES_NORMAL) && $dtrescisao) {
            
            $condfispes = $this->entityManager->getRepository(Repositories::GPE_CONDFISPES)->findOneBy(array('nrorg' => $nrorg,
            'nrcondfispes' => $pessoah->getNrcondfispes()));
            
            $dscondfispes = is_object($condfispes) ? $condfispes->getDscondfispes() : 'de deficiente';

            return $dscondfispes;
        }
        return false;
    }
    
    /*O empregado dispensado, sem justa causa, no período de 30 dias que antecede
      a data de sua correção salarial (dissídio), terá direito à indenização.*/
    public function verificaDissidio($nrorg, $nrvinculom, $dtcompetencia, $dtrescisao,$nrtpdemissao){
        
        $vinculoh = $this->retornaUltimoHistoricoCompetencia($nrorg, $nrvinculom, $dtcompetencia);
        $dtbasesindical = null;
        $nrestruturasind = $vinculoh->getNrestrutsind();
        $estruturasindical = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findOneBy(array('nrorg' => $nrorg,
                    'nrestruturam' => $nrestruturasind));
        if($estruturasindical) {
            $dtbasesindical = $estruturasindical->getDtbasesindical();
        }
        if($dtbasesindical && $dtrescisao) {
            $dtrescisao = Util::datadeString(substr($dtrescisao, 0, 10), true);
            $intervalo = $dtrescisao->diff($dtbasesindical);
            $qtdias =  (int)$intervalo->format('%R%a');
            $intervalo1 = $dtbasesindical->diff($dtrescisao);
            $qtdias1 =  (int)$intervalo1->format('%R%a');
            if ($qtdias <= 30
                && $qtdias1 <= 30 && $nrtpdemissao == FpaTpdemissao::SEM_JUSTACAUSA) {
                return true;
            }
        } 
        return false;
    }
    
    public function verificaMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, $dtmescompetenc, $nrestruturam, $cdmatriculaesocialante){
        if($cdmatriculaesocialante != -1){    
            $retorno = array();
            if(isset($nrvinculom)){
                $envio2200 = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEvento($nrorg, 'S-2200', $nrestruturam, 1, 'S', $nrvinculom, null, null, 'N');
                $envio2300 = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->getEnviosPorNomeEvento($nrorg, 'S-2300', $nrestruturam, 1, 'S', $nrvinculom, null, null, 'N');
            }else{
                $envio2200 = false;
                $envio2300 = false;
            }
            $matriculaRepetida = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), $nrestruturam);
            if(!empty($envio2200)  || !empty($envio2300) || !empty($matriculaRepetida)){
                $retorno['message'] = 'Matrícula do ESocial não pode ser alterada! Verificar a existência de envios (S-2200, S-2300) ou outro vínculo com o mesmo código.';
                $retorno['type'] = 'ALERT';
                return $retorno;
            }else{
                $parametro = $this->entityManager->getRepository(Repositories::ESO_PARAMETRO)->findOneBy(array('nrestruturam' => $nrestruturam, 'nrorg' => $nrorg));
                if($parametro && $parametro->getIdutilizamatric() == 'S' && !empty($cdmatriculaesocial)){    
                    $qtdeDigitos = $parametro->getNrqtdedigmatric();
                    $prefixo = $parametro->getDsprefixomatric();
                    $prefixMatriculaNova = $prefixo ? str_split($cdmatriculaesocial, strlen($prefixo)): null;
                    $prefixMatriculaNova = is_array($prefixMatriculaNova) ? $prefixMatriculaNova[0] : $prefixMatriculaNova;
                    
                    if($prefixo == $prefixMatriculaNova && (strlen($cdmatriculaesocial) == (strlen($prefixo)+$qtdeDigitos))){
                        $retorno['type'] = 'SUCESS';
                        return $retorno;
                    }else{
            
                        $retorno['message'] = 'Matrícula do ESocial incompatível com o padrão da estrutura. (prefixo:'.$prefixo.' | qtde. dígitos: '.$qtdeDigitos.')';
                        $retorno['type'] = 'ERROR';
                        return $retorno;
                    }
                } else {
                    $retorno['type'] = 'SUCESS';
                    return $retorno;
                }    
            }
        }else{
            $retorno['type'] = 'SUCESS';
            return $retorno;
        }
    }
    
    public function setSeguroVinculo($nrvinculom, $idsegdesempvinc){
         try {
            $this->entityManager->beginTransaction();
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtcompetencia = $this->hcmParameters->getDtCompetencia();
            $gpevinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $nrorg));
	        $gpevinculom->setIdsegdesempvinc($idsegdesempvinc);
	        
            $this->entityManager->flush();
            $this->entityManager->commit();
            
            return $gpevinculom;
            
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    public function getFichaFinanceira($parameters, $conditions){	 
        return $this->vinculoRepository->getFichaFinanceira($parameters, $conditions);	 
    }
    
    public function getVinculosMatriculaEsocial($parameters, $conditions){
        return $this->vinculoRepository->getVinculosMatriculaEsocial($parameters, $conditions);
    }
    
    /**
     * Função responsável por retornar array de vínculos com  historico agregado.
     */
    public function retornaOnlyObjectVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos = false, $ds_inicio = null, $ds_fim = null, $dsOrdem = null, $carregaHistorico = false, $nrEstrutura = null,
                                               $verificaConfidencial = true, $nrTipoVinculom = null, $nrVinculoEmpreg = null, $nrEstrutSind = null) {

        if(!empty($ds_inicio) && is_array($ds_inicio)){
            $ds_inicio = implode(',', $ds_inicio);
        }
        if(!empty($ds_fim) && is_array($ds_fim)){
            $ds_fim = implode(',', $ds_fim);
        }
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim    = trim(strtoupper($ds_fim));

        // define primeiro dia do mês competência
        $primeiroDiaAno = DateUtil::subtraiIntervalo($this->hcmParameters->getPrimeiroDiaDoAno(), 1, DateUtil::ANO)->format('d/m/Y');
        $dtUltDiaMes = $this->hcmParameters->getUltimoDiaMes();
        return $this->vinculoRepository->retornaOnlyObjectVinculosCompetencia($nrOrg, $mesCompetencia, $rescindidos, $primeiroDiaAno, $dtUltDiaMes, $ds_inicio, $ds_fim, $dsOrdem, $carregaHistorico, $nrEstrutura,
                                               $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind);   

    }
    
    public function getVinculoNr($parameters) {
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $parameters["P_DTMESCOMPETENC"];
        $nrvinculom     = $parameters["P_NRVINCULOM"];
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT VM.NMVINCULOM,
              PM.NRPARCNEGOCIO,
              PH.NRPESSOA,
              PH.NMPESSOA,
              EN.NRCEPENDERECO,
              EN.CDTIPOENDERECO,
              PA.NMPAIS,
              EN.SGESTADO,
              MU.NMMUNICIPIO,
              EN.CDBAIRRO,
              EN.CDLOGRADOURO,
              EN.NMBAIRROENDERECO,
              EN.DSCOMPLEENDERECO,
              EN.DSENDERECO,
              EN.NRIMOVELENDERECO,
              EN.DSREFERENCIAENDE,
              GR.NMGRAUINSTR,
              BC.CDBANCO,
              BC.NMBANCO,
              AG.NMAGENCIA,
              AG.CDAGENCIA,
              TPC.NRTPCONTCORR,
              TPC.DSTPCONTCORR,
              CT.CDCONTCORR,
              PH.NRCERTCASAPES,
              PH.DTCASAPESSOA,
              PH.DTHABCARTPES,
              PH.DSCATEGHABCART,
              PH.DTVALHABCARTPES,
              PH.NRCARTHABPES,
              VM.DTADMISSAOVINC,
              PH.NRCPFPESSOA,
              GPE_ALTEESCALA.DTINIESCALA,
              GPE_ESCALATRABH.NMESCALATRABH,
              GPE_TURNO.DSTURNO,
              FC.NMFORMACOMU,
              CP.CDFORMACOMU,
              CP.CDPREFIXCOMUPARC,
              CP.CDCOMUNICAPARC,
              GAS.DTALTESALARIO,
              GAS.NRTPMODALIDSAL,
              GAS.VRSALARIO,
              FPA_TPMODALIDSAL.NMTPMODALIDSAL,
              GAS.IDTPSALARIO,
              GPE_TIPOSALARIO.DSTIPOSALARIO,
              GPE_ALTEOCUPACAO.NROCUPACAOM,
              OH.NMOCUPACAOH,
              GPE_ALTEOCUPACAO.DTINIOCUPACAO,
              GPE_ALTEOCUPACAO.NRMOTIVOOCUPA,
              GPE_MOTIVOOCUPA.NMMOTIVOOCUPA,
              GPE_ALTEOCUPACAO.NRTIPOOCUPACAO,
              GPE_TIPOOCUPACAO.NMTIPOOCUPACAO,
              VH.*
            FROM GPE_VINCULOM VM

            JOIN GPE_VINCULOH VH
                ON VH.NRORG           = VM.NRORG
                AND VH.NRVINCULOM     = VM.NRVINCULOM
                AND VH.DTMESCOMPETENC =
                  (SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_VINCULOH
                  WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                  AND NRORG             = VH.NRORG
                  AND NRVINCULOM        = VH.NRVINCULOM
                  )

            LEFT JOIN GPE_PESSOAH PH
                ON PH.NRPESSOA        = VM.NRPESSOA
                AND PH.NRORG          = VM.NRORG
                AND PH.DTMESCOMPETENC =
                  (SELECT MAX(DTMESCOMPETENC)
                  FROM GPE_PESSOAH
                  WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                  AND NRORG             = PH.NRORG
                  AND NRPESSOA          = PH.NRPESSOA
                  )

            LEFT JOIN GPE_PESSOA PM
                ON PH.NRPESSOA = PM.NRPESSOA
                AND PH.NRORG   = PM.NRORG

            LEFT JOIN ENDERECOPARC EN
                ON PM.NRPARCNEGOCIO = EN.NRPARCNEGOCIO

            LEFT JOIN PAIS PA
                ON EN.CDPAIS = PA.CDPAIS
    
            LEFT JOIN MUNICIPIO MU
                ON EN.CDMUNICIPIO = MU.CDMUNICIPIO
        
            LEFT JOIN GPE_GRAUINSTR GR
                ON GR.NRGRAUINSTR = PH.NRGRAUINSTR

            LEFT JOIN CONTCORRPARC CT
                ON CT.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
    
            LEFT JOIN BANCO BC
                ON BC.CDBANCO = CT.CDBANCO
        
            LEFT JOIN AGENCIA AG
                ON AG.CDAGENCIA = CT.CDAGENCIA

            LEFT JOIN TPCONTCORR TPC
                ON TPC.NRTPCONTCORR      = CT.NRTPCONTCORR
                
                
            /* ESCALA TRABALHO*/
            LEFT JOIN 
                GPE_ALTEESCALA 
                    ON GPE_ALTEESCALA.NRESCALATRABM = VH.NRESCALATRABM
                    AND GPE_ALTEESCALA.NRORG = VH.NRORG
                    AND GPE_ALTEESCALA.NRVINCULOM = VH.NRVINCULOM
                    AND GPE_ALTEESCALA.DTINIESCALA = (
                                                            SELECT
                                                                MAX(AE.DTINIESCALA) 
                                                        FROM
                                                            GPE_ALTEESCALA AE 
                                                        WHERE
                                                            AE.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM 
                                                            AND AE.NRVINCULOM = GPE_ALTEESCALA.NRVINCULOM
                                                            AND AE.NRORG = GPE_ALTEESCALA.NRORG 
                                                            AND AE.DTINIESCALA <=:P_DTMESCOMPETENC 
                                                        )    
            LEFT JOIN
                GPE_ESCALATRABM 
                    ON GPE_ESCALATRABM.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM 
                    AND (
                        GPE_ESCALATRABM.NRORG = GPE_ALTEESCALA.NRORG 
                        OR GPE_ESCALATRABM.NRORG =:P_NRORG_PADRAO 
                    ) 
            LEFT JOIN
                GPE_ESCALATRABH 
                    ON GPE_ESCALATRABH.NRESCALATRABM = GPE_ESCALATRABM.NRESCALATRABM 
                    AND (
                        GPE_ESCALATRABH.NRORG = GPE_ESCALATRABM.NRORG 
                        OR GPE_ESCALATRABH.NRORG =:P_NRORG_PADRAO 
                    ) 
                    AND GPE_ESCALATRABH.DTMESCOMPETENC = (
                                                                SELECT
                                                                        MAX(EH.DTMESCOMPETENC) 
                                                                FROM
                                                                    GPE_ESCALATRABH EH 
                                                                WHERE
                                                                    EH.NRESCALATRABM = GPE_ESCALATRABH.NRESCALATRABM 
                                                                    AND EH.NRORG = GPE_ESCALATRABH.NRORG 
                                                                    AND EH.DTMESCOMPETENC <=:P_DTMESCOMPETENC 
                                                          ) 
            LEFT JOIN
                GPE_TURNO 
                    ON GPE_TURNO.NRTURNO = GPE_ALTEESCALA.NRTURNO 
                    AND GPE_TURNO.NRORG = GPE_ALTEESCALA.NRORG 
                    AND GPE_TURNO.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM             
            
            /*FORMA COMUNICAÇÃO*/
            LEFT JOIN COMUNICAPARC CP 
                ON CP.NRORG = VH.NRORG
                AND CP.NRPARCNEGOCIO = PM.NRPARCNEGOCIO
                AND CP.CDFORMACOMU = '01'
        
            LEFT JOIN FORMACOMU FC 
                ON CP.CDFORMACOMU = FC.CDFORMACOMU 
                AND(
                    FC.NRORG = CP.NRORG 
                    OR FC.NRORG = :P_NRORG_PADRAO
                ) 
                
            /*ALTERAÇÃO SALARIO*/    
            LEFT JOIN GPE_ALTESALARIO GAS
                ON GAS.NRORG = VM.NRORG
                AND GAS.NRVINCULOM = VM.NRVINCULOM
                AND GAS.DTALTESALARIO = (
                                            SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                            WHERE GAS.NRORG = GPE_ALTESALARIO.NRORG
                                            AND GAS.NRVINCULOM = GPE_ALTESALARIO.NRVINCULOM
                                        )
            
            LEFT JOIN
                FPA_TPMODALIDSAL 
                    ON FPA_TPMODALIDSAL.NRTPMODALIDSAL = GAS.NRTPMODALIDSAL 
                    AND (
                        FPA_TPMODALIDSAL.NRORG = GAS.NRORG 
                        OR FPA_TPMODALIDSAL.NRORG = :P_NRORG_PADRAO
                    ) 
            LEFT JOIN
                GPE_TIPOSALARIO 
                    ON GPE_TIPOSALARIO.CDTIPOSALARIO = GAS.IDTPSALARIO 
                    AND (
                        GPE_TIPOSALARIO.NRORG = GAS.NRORG 
                        OR GPE_TIPOSALARIO.NRORG = :P_NRORG_PADRAO
                    )                     


            /*OCUPAÇÃO*/
            LEFT JOIN GPE_ALTEOCUPACAO 
                ON GPE_ALTEOCUPACAO.NRVINCULOM = VM.NRVINCULOM
                AND GPE_ALTEOCUPACAO.NRORG = VM.NRORG
            JOIN
                GPE_OCUPACAOH OH 
                    ON OH.NROCUPACAOM = GPE_ALTEOCUPACAO.NROCUPACAOM 
                    AND OH.NRORG = GPE_ALTEOCUPACAO.NRORG 
                    AND OH.DTMESCOMPETENC = (
                                                SELECT
                                                    MAX(OH1.DTMESCOMPETENC) 
                                                FROM
                                                    GPE_OCUPACAOH OH1 
                                                WHERE
                                                    OH1.NROCUPACAOM = OH.NROCUPACAOM 
                                                    AND OH1.NRORG = OH.NRORG 
                                                    AND OH1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                            )
            LEFT JOIN
                GPE_MOTIVOOCUPA 
                    ON GPE_MOTIVOOCUPA.NRMOTIVOOCUPA = GPE_ALTEOCUPACAO.NRMOTIVOOCUPA 
                    AND (
                        GPE_MOTIVOOCUPA.NRORG = GPE_ALTEOCUPACAO.NRORG 
                        OR GPE_MOTIVOOCUPA.NRORG = :P_NRORG_PADRAO
                    )

            LEFT JOIN
                GPE_TIPOOCUPACAO 
                    ON GPE_TIPOOCUPACAO.NRTIPOOCUPACAO = GPE_ALTEOCUPACAO.NRTIPOOCUPACAO 
                    AND (
                        GPE_TIPOOCUPACAO.NRORG = GPE_ALTEOCUPACAO.NRORG 
                        OR GPE_TIPOOCUPACAO.NRORG = :P_NRORG_PADRAO
                    )                     


            WHERE VM.NRORG           = :P_NRORG
                AND ( VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                OR VM.DTRESCISAOVINC    IS NULL )
                AND (VM.NRVINCULOM       = :P_NRVINCULOM)
        ");

        $query->execute(
            array(
                "P_NRORG"                 => $nrorg,
                "P_NRORG_PADRAO"          => $this->hcmParameters->getNrorgPadrao(),
                "P_DTMESCOMPETENC"        => $dtmescompetenc,
                "P_NRVINCULOM"            => $nrvinculom
            )
        );

        $vinculo = $query->fetchAll();
        return $vinculo;
    }
    
    public function alteraBeneficiários($nrvinculom, $dtRescisaoVinc, $dtRescisaoRow) {
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $modificaBenef = false;
        
        if($dtRescisaoVinc != $dtRescisaoRow) {
            $modificaBenef = true;
        } else {
            $modificaBenef = false;
        }
        
        if($modificaBenef) {
           $this->vinculoRepository->alteraBeneficiários($nrvinculom, $dtRescisaoRow, $nrorg);
        }
    }
    
    public function updateConvocacaoTrabalhoIntermitente ($parameters){
        return $this->vinculoRepository->updateConvocacaoTrabalhoIntermitente($parameters);
    }
    
    public function existeMesmoPeriodoConvocacaoTrabalhoIntermitente ($parameters){
        $result = $this->vinculoRepository->existeMesmoPeriodoConvocacaoTrabalhoIntermitente($parameters);
        if( count($result) > 0 ){
            if( array_key_exists('NRCONVOCACAO', $parameters) ){
                if( $result[0]['NRCONVOCACAO'] == $parameters['NRCONVOCACAO'] ){
                    return false;
                }
            }
            return true;
        }
    }
    
    public function updateComposicaoValeTransporte ($parameters, $flag = null){
        $dataInicio = implode( '', array_reverse(explode('/', substr($parameters['DTINICONVOCACAO'], 0, 10))) );
        $dataFim = implode( '', array_reverse(explode('/', substr($parameters['DTFIMCONVOCACAO'], 0, 10))) );
        $vinculoh = $this->retornaUltimoHistoricoCompetencia($parameters['NRORG'], $parameters['NRVINCULOM'], $parameters['DTINICONVOCACAO']);
        if( substr($vinculoh->getDtmescompetenc()->format('d/m/Y'), 3,7) == substr($parameters['DTINICONVOCACAO'], 3,7) ){
            $totalDiasComposicaoVT = $vinculoh->getNrdiasapurvales();
        } else {
            $totalDiasComposicaoVT = 0;
        } 
        
       
        $periodoAntigo = $parameters['PERIODOANTIGO'];
        $periodoNovo = $dataFim - $dataInicio + 1;
        
        if( $flag == "update" ){
            $parameters['NRDIASAPURVALES'] = $totalDiasComposicaoVT - $periodoAntigo + $periodoNovo;
        } else if( $flag == "delete" ){
            $parameters['NRDIASAPURVALES'] = $totalDiasComposicaoVT - $periodoNovo;
        }

        if( substr($vinculoh->getDtmescompetenc()->format('d/m/Y'), 3,7) == substr($parameters['DTINICONVOCACAO'], 3,7) ){
            $vinculoh->setNrdiasapurvales($parameters['NRDIASAPURVALES']);
            $vinculoh->setNrorgultatu($parameters['NRORG']);
        	$vinculoh->setCdoperultatu($this->hcmParameters->getCdoperador());
        	$vinculoh->setDtultatu(DateUtil::getDataAtual());
        	$this->entityManager->persist($vinculoh);
            $this->entityManager->flush($vinculoh);
        } else {
            $dtmescompetenc = str_split(DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($parameters['DTINICONVOCACAO']))->__toString(), 10)[0];
            $dtmescompetenc = DateUtil::getDataDeString(substr($dtmescompetenc, 0, 10));
            $dtmescompetenc = DateUtil::truncateData($dtmescompetenc);
            $alteOcupacao = $this->entityManager->getRepository(Repositories::GPE_ALTEOCUPACAO)->findOneBy(array('nrvinculom' => $parameters['NRVINCULOM'], 'nrorg' => $parameters['NRORG']));
            
            $this->novoHistorico($parameters['NRORG'], $parameters['NRVINCULOM'], $dtmescompetenc, $vinculoh->getNrsitufuncm(), $alteOcupacao->getNrocupacaom(), $vinculoh->getNrescalatrabm(),
                                  $vinculoh->getIdcontribuisind(), $vinculoh->getNrdependir(), $vinculoh->getNrdependsfam(), $vinculoh->getDtbaseferias(), $vinculoh->getIdremuneracao(),
                                  $vinculoh->getIdtppagamento(), $vinculoh->getNrestrutlegal(), $vinculoh->getNrestrutgeren(), $vinculoh->getNrtpmodalidsal(), $vinculoh->getIdmultvinc(),
                                  $vinculoh->getNrfuncao(), $vinculoh->getCdcontribindividual(), $vinculoh->getNrtpmovtransfm(), $vinculoh->getDtfimcontrdetermin(),
                                  $vinculoh->getNrestrutsind(), $vinculoh->getConfidencial());

            $vinculoh = $this->retornaUltimoHistoricoCompetencia($parameters['NRORG'], $parameters['NRVINCULOM'], $parameters['DTINICONVOCACAO']);
            $vinculoh->setNrdiasapurvales($parameters['NRDIASAPURVALES']);
            $this->entityManager->persist($vinculoh);
            $this->entityManager->flush($vinculoh);
        }
        return 0;
    }
    
    public function setCadastroAutomaticoConvocacaoTrabalhoIntermitente ($parameters){
        $this->vinculoRepository->setCadastroAutomaticoConvocacaoTrabalhoIntermitente($parameters);
        $this->updateComposicaoValeTransporte($parameters, "update");
        return 0;
    }
    
    public function getSituacaoVinculoEsocial($parameters, $conditions){	 
        return $this->vinculoRepository->getSituacaoVinculoEsocial($parameters, $conditions);	 
    }
}