CREATE OR REPLACE PROCEDURE ALTERA_DEPRECIACAO(
   P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDGRUPOBEM IN VARCHAR,
   P_CDBEM IN VARCHAR,
   P_CDDEPREC IN VARCHAR,
   P_CDOPERADOR IN VARCHAR)

AS
   V_TAXAANUAL NUMBER;
   V_TAXADEPREC NUMBER;
   V_VRDEPREC NUMBER;

   V_TAXAANUALFIS NUMBER;
   V_TAXADEPRECFIS NUMBER;
   V_VRDEPRECFIS NUMBER;

   V_COEFICIENTEDEP NUMBER;
   V_COEFICIENTEDEPFIS NUMBER;

   V_MESESFALTADEP NUMBER;
   V_MESESFALTADEPFIS NUMBER;

   V_VRCORRIGECTB NUMBER;
   V_VRCORRIGEFIS NUMBER;

   V_VRFALTADEPCTB NUMBER;
   V_VRFALTADEPFIS NUMBER;
BEGIN
  /*  Identificadores do campo IDTIPOOPERBEM, tabela TIPOOOPERBEM:
     01 - IMOBILIZACAO
     02 - DEPRECICACAO
     03 - TRANSFERENCIA_FISICA
     04 - BAIXA_VENDA
     05 - BAIXAS_DIVERSAS
     06 - REAVALIACAO
     07 - SUSPENSAO_DEPRECIACAO
     08 - RETORNO_DEPRECIACAO
     09 - TRANSFERENCIA_CONTABIL

     Identificadores do campo IDIDENTCAPAOPER, tabela CAPAOPERABEM
     Os mesmos da tabela TIPOOPERBEM, incluindo:
     10 - LIVRE PARA USO

     Identificadores do campo tabela OPERACAOBEM
     Os mesmos da tabela TIPOOPERBEM, incluindo:
     10 - LIVRE PARA USO
     
     Identificadores do campo IDSITCAPAOPER, tabela CAPAOPERABEM
     1 - Pendete
     2 - Processado
     3 - Cancelado
     4 - Lote Gerado

     Identificador campo IDSITOPERABEM, abela OPERACAOBEM
     1 - No Contabilizado
     2 - Lote Gerado
     3 - Cancelado 
     
     Parametrizao da depreciao por empresa, campo IDTPCALCDEPREC, tabela PARAMEMPRESA
      1 - Deprecia integral no mesmo ms de aquisio
      2 - Deprecia proporcional no mesmo ms de aquisio
      3 - Avalia o dia da aquisio para determinar o ms de incio da depreciao
          Verifica o campo NRDIAAQUIBEM
      4 - Deprecia integral no ms seguinte  aquisio
     
     IDTIPOFECHAOPER
      F - Fiscal
      C - Contbil */

   SELECT NRCOETIPODEP, NRCOETIPODEPFIS
     INTO V_COEFICIENTEDEP, V_COEFICIENTEDEPFIS
     FROM TIPODEPREC
    WHERE CDEMPRESA = P_CDEMPRESA
      AND CDTIPODEPREC = P_CDDEPREC;

   /* FILTRA OS BENS A TEREM SUAS DEPRECIACOES ALTERADAS */
   FOR C_BENS IN
      (SELECT B.CDEMPRESA, B.CDGRUPOBEM, B.CDBEMATIVO, B.IDNOVOUSABEM, B.QTMESUSOBEM,
              B.VRTAXANODEPBEM, B.VRTXANODEPBEMFI, B.VRFALTDEPBEMFIS, B.VRFALTADEPBEM,
              NVL(G.IDDEPRECIAGRBEM, 'N') IDDEPRECIAGRBEM, NVL(G.QTMESDEPREGRBEM, 1) QTMESDEPREGRBEM,
              NVL(G.IDDEPRECIAFISC, 'N') IDDEPRECIAFISC,  NVL(G.QTMESDEPREFISC,1) QTMESDEPREFISC,
              B.VRAQUIBEMATIVO, B.VRAQUIBEMFISC,
              NVL(T.NRCOETIPODEP, 0) NRCOETIPODEP, NVL(T.NRCOETIPODEPFIS, 0) NRCOETIPODEPFIS, B.VRINIDEPBEM, B.VRINIDEPRECFIS, B.VRJADEPBEMATIVO,
              B.CDTIPODEPREC, B.VRJADEPBEMFISC, B.QTMESJADEPBEM, B.QTMESJADEPBEMFI
         FROM BEMATIVO B, GRUPOBEM G, TIPODEPREC T
        WHERE B.CDEMPRESA = P_CDEMPRESA
          AND ((INSTR(P_CDGRUPOBEM, B.CDGRUPOBEM) > 0) OR ('T' = P_CDGRUPOBEM))
          AND ((INSTR(P_CDBEM, B.CDBEMATIVO) > 0) OR ('T' = P_CDBEM))
          AND B.CDEMPRESA = G.CDEMPRESA
          AND B.CDGRUPOBEM = G.CDGRUPOBEM
          AND B.CDTIPODEPREC = T.CDTIPODEPREC) LOOP

      /* SETA AS VARIAVEIS COM NULO PARA SEREM USADAS NO NVL DO UPDATE (CASO NAO DEPRECIEM CONTABILMENTE) */
      V_TAXAANUAL := NULL;
      V_MESESFALTADEP := NULL;
      
      /* CALCULA AS TAXAS ANUAIS PARA O NOVO TIPO DE DEPRECIACAO */
      IF C_BENS.IDDEPRECIAGRBEM = 'S' THEN /* DEPRECIACAO CONTABIL */
         IF C_BENS.VRFALTADEPBEM > 0 THEN
            IF V_COEFICIENTEDEP <> C_BENS.NRCOETIPODEP THEN
               V_TAXADEPREC := (100 / C_BENS.QTMESDEPREGRBEM) / 100; /* TAXA A SER DEPRECIADA MENSALMENTE */
               V_TAXAANUAL := ROUND(V_TAXADEPREC * 12 * 100, 2); /* TAXA ANUAL */

               /* CALCULO DO VALOR A SER DEPRECIADO MENSALMENTE */
               IF V_COEFICIENTEDEP = 1 THEN
                  /* VOLTA A DEPRECIACAO AO VALOR NORMAL */
                  V_VRDEPREC := C_BENS.VRINIDEPBEM * V_TAXADEPREC;
               ELSIF V_COEFICIENTEDEP = 1.5 THEN
                  V_VRDEPREC := C_BENS.VRINIDEPBEM * (V_TAXADEPREC * 1.5);
               ELSIF V_COEFICIENTEDEP = 2 THEN
                  /* DEPRECIACAO ACELERADA -> TAXA x2 */
                  V_VRDEPREC := C_BENS.VRINIDEPBEM * (V_TAXADEPREC * 2);
               END IF;
               V_VRDEPREC := ROUND(V_VRDEPREC, 2);
               /* A SITUACAO ABAIXO PODE ACONTECER QUANDO, POR EXEMPLO, SE FOR O LTIMO MS DE DEPRECIAO
                  E APS MULTIPLIACAR O VALOR DE DEPRECIAO PELO COEFICIENTE O VALOR A SER DEPRECIADO
                  ULTRAPASSA O VALOR RESIDUAL DE DEPRECIAO.
                  CASO ISSO ACONTEA, A QUANTIDADE DE MESES RESTANTES PARA DEPRECIAR DEVE SER APENAS 1,
                  POIS O VALOR RESIDUAL DEVER SER DEPRECIADO NA PARCELA RESTANTE */
               IF V_VRDEPREC > C_BENS.VRFALTADEPBEM THEN
                  V_VRDEPREC := C_BENS.VRINIDEPBEM - C_BENS.VRJADEPBEMATIVO;
               END IF;

               V_MESESFALTADEP := 0;
               V_MESESFALTADEP := TRUNC(C_BENS.VRFALTADEPBEM / V_VRDEPREC); /* CALCULA A QUANTIDADE DE MESES A SEREM DEPRECIADOS */

               /* CALCULA SE DEPOIS QUE DEPRECIAR TODOS OS MESES, TER ALGUM VALOR RESIDUAL,
                  CASO TENHA,  PRECISO DEPRECIAR O RESTO EM MAIS UM MS */
               IF V_MESESFALTADEP < C_BENS.QTMESDEPREGRBEM THEN
                  IF V_MESESFALTADEP < (C_BENS.VRFALTADEPBEM / V_VRDEPREC) THEN
                     IF ((C_BENS.VRFALTADEPBEM / V_VRDEPREC) - V_MESESFALTADEP) >= 0.5 THEN
                        V_MESESFALTADEP := V_MESESFALTADEP + 1;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;


      /* SETA AS VARIAVEIS COM NULO PARA SEREM USADAS NO NVL DO UPDATE (CASO NAO DEPRECIEM FISCALMENTE) */
      V_MESESFALTADEPFIS := NULL;
      V_TAXAANUALFIS := NULL;

      /* CALCULA AS TAXAS ANUAIS PARA O NOVO TIPO DE DEPRECIACAO */
      IF C_BENS.IDDEPRECIAFISC = 'S' THEN /* DEPRECIACAO CONTABIL */
         IF C_BENS.VRFALTDEPBEMFIS > 0 THEN
            IF V_COEFICIENTEDEPFIS <> C_BENS.NRCOETIPODEPFIS THEN
               V_TAXADEPRECFIS := (100 / C_BENS.QTMESDEPREFISC) / 100; /* TAXA A SER DEPRECIADA MENSALMENTE */
               V_TAXAANUALFIS := ROUND(V_TAXADEPRECFIS * 12 * 100, 2); /* TAXA ANUAL */

               /* CALCULO DO VALOR A SER DEPRECIADO MENSALMENTE */
               IF V_COEFICIENTEDEPFIS = 1 THEN
                  /* VOLTA A DEPRECIACAO AO VALOR NORMAL */
                  V_VRDEPRECFIS := C_BENS.VRINIDEPRECFIS * V_TAXADEPRECFIS;
               ELSIF V_COEFICIENTEDEPFIS = 1.5 THEN
                  V_VRDEPRECFIS := C_BENS.VRINIDEPRECFIS * (V_TAXADEPRECFIS * 1.5);
               ELSIF V_COEFICIENTEDEPFIS = 2 THEN
                  /* DEPRECIACAO ACELERADA -> TAXA x2 */
                  V_VRDEPRECFIS := C_BENS.VRINIDEPRECFIS * (V_TAXADEPRECFIS * 2);
               END IF;
               V_VRDEPRECFIS := ROUND(V_VRDEPRECFIS, 2);
               /* A SITUACAO ABAIXO PODE ACONTECER QUANDO, POR EXEMPLO, SE FOR O LTIMO MS DE DEPRECIAO
                  E APS MULTIPLIACAR O VALOR DE DEPRECIAO PELO COEFICIENTE O VALOR A SER DEPRECIADO
                  ULTRAPASSA O VALOR RESIDUAL DE DEPRECIAO.
                  CASO ISSO ACONTEA, A QUANTIDADE DE MESES RESTANTES PARA DEPRECIAR DEVE SER APENAS 1,
                  POIS O VALOR RESIDUAL DEVER SER DEPRECIADO NA PARCELA RESTANTE */
               IF V_VRDEPRECFIS > C_BENS.VRFALTDEPBEMFIS THEN
                  V_VRDEPRECFIS := C_BENS.VRINIDEPRECFIS - C_BENS.VRJADEPBEMFISC;
               END IF;

               V_MESESFALTADEPFIS := 0;
               V_MESESFALTADEPFIS := TRUNC(C_BENS.VRFALTDEPBEMFIS / V_VRDEPRECFIS); /* CALCULA A QUANTIDADE DE MESES A SEREM DEPRECIADOS */

               /* CALCULA SE DEPOIS QUE DEPRECIAR TODOS OS MESES, TER ALGUM VALOR RESIDUAL,
                  CASO TENHA,  PRECISO DEPRECIAR O RESTO EM MAIS UM MS */
               IF V_MESESFALTADEPFIS < C_BENS.QTMESDEPREFISC THEN
                  IF V_MESESFALTADEPFIS < (C_BENS.VRFALTDEPBEMFIS / V_VRDEPRECFIS) THEN
                     IF ((C_BENS.VRFALTDEPBEMFIS / V_VRDEPRECFIS) - V_MESESFALTADEPFIS) >= 0.5 THEN
                        V_MESESFALTADEPFIS := V_MESESFALTADEPFIS + 1;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;

      IF ((V_COEFICIENTEDEPFIS <> C_BENS.NRCOETIPODEPFIS) OR
          (V_COEFICIENTEDEP <> C_BENS.NRCOETIPODEP)) THEN
         INSERT INTO LOGDEPRECAUT
            (CDEMPRESA, CDBEMATIVO, DTALTERACAO, CDDEPRECANTIGO, NRCOECTBANTIGO,
             NRCOEFISANTIGO, CDDEPRECNOVO, NRCOECTBNOVO, NRCOEFISNOVO, CDOPERADOR)
         VALUES
            (C_BENS.CDEMPRESA, C_BENS.CDBEMATIVO, SYSDATE, C_BENS.CDTIPODEPREC, C_BENS.NRCOETIPODEP,
             C_BENS.NRCOETIPODEPFIS, P_CDDEPREC, V_COEFICIENTEDEP, V_COEFICIENTEDEPFIS, P_CDOPERADOR);

         V_VRCORRIGECTB := NULL;
         V_VRFALTADEPCTB := NULL;
         IF C_BENS.QTMESJADEPBEM = 0 THEN
            V_VRCORRIGECTB := 0;
            V_VRFALTADEPCTB := C_BENS.VRINIDEPBEM;
         END IF;

         V_VRCORRIGEFIS := NULL;
         V_VRCORRIGEFIS := NULL;
         IF C_BENS.QTMESJADEPBEMFI = 0 THEN
            V_VRCORRIGEFIS := 0;
            V_VRFALTADEPFIS := C_BENS.VRINIDEPRECFIS;
         END IF;

         /* ATUALIZA OS VALORES REFERENTES AS DEPRECIACOES */
         UPDATE BEMATIVO
            SET QTMESFALTADBEM =  NVL(V_MESESFALTADEP, QTMESFALTADBEM),
                VRTAXANODEPBEM = NVL(V_TAXAANUAL, VRTAXANODEPBEM),
                QTMESFALTDBEMFIS = NVL(V_MESESFALTADEPFIS, QTMESFALTDBEMFIS),
                VRTXANODEPBEMFI = NVL(V_TAXAANUALFIS, VRTXANODEPBEMFI),
                CDTIPODEPREC = P_CDDEPREC,
                VRJADEPBEMATIVO = NVL(V_VRCORRIGECTB, VRJADEPBEMATIVO),
                VRJADEPBEMFISC = NVL(V_VRCORRIGEFIS, VRJADEPBEMFISC),
                VRFALTADEPBEM = NVL(V_VRFALTADEPCTB, VRFALTADEPBEM),
                VRFALTDEPBEMFIS = NVL(V_VRFALTADEPFIS, VRFALTDEPBEMFIS)
           WHERE CDEMPRESA = C_BENS.CDEMPRESA
             AND CDBEMATIVO = C_BENS.CDBEMATIVO;
      END IF;
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ALTERA_DEPRECIACAO','5.85.xxx'); END;
/
